@@ -1,16 +1,20 @@
 package Crypt::SSLeay;
 
-use Crypt::SSLeay::X509;
-
 use strict;
-use vars qw(@ISA $VERSION %CIPHERS);
+use vars '$VERSION';
+$VERSION = '0.54';
 
-require DynaLoader;
-
-@ISA = qw(DynaLoader);
-$VERSION = '0.53';
-
-bootstrap Crypt::SSLeay $VERSION;
+eval {
+    require XSLoader;
+    XSLoader::load('Crypt::SSLeay', $VERSION);
+    1;
+}
+or do {
+    require DynaLoader;
+    use vars '@ISA'; # not really locally scoped, it just looks that way
+    @ISA = qw(DynaLoader);
+    bootstrap Crypt::SSLeay $VERSION;
+};
 
 use vars qw(%CIPHERS);
 %CIPHERS = (
@@ -27,6 +31,7 @@ use vars qw(%CIPHERS);
    'DES-CFB-M1'   => "56 bit CFB64 DES encryption with a one byte MD5 MAC",
 );
 
+use Crypt::SSLeay::X509;
 
 # A xsupp bug made this nessesary
 sub Crypt::SSLeay::CTX::DESTROY  { shift->free; }
@@ -39,17 +44,44 @@ __END__
 
 =head1 NAME
 
-Crypt::SSLeay - OpenSSL glue that provides LWP https support
+Crypt::SSLeay - OpenSSL support for LWP
 
 =head1 SYNOPSIS
 
   lwp-request https://www.example.com
 
   use LWP::UserAgent;
-  my $ua = new LWP::UserAgent;
-  my $req = new HTTP::Request('GET', 'https://www.example.com');
+  my $ua  = LWP::UserAgent->new;
+  my $req = HTTP::Request->new('GET', 'https://www.example.com/');
   my $res = $ua->request($req);
-  print $res->code."\n";
+  print $res->content, "\n";
+
+=head1 DESCRIPTION
+
+This document describes C<Crypt::SSLeay> version 0.54, released
+2007-04-13.
+
+This perl module provides support for the https protocol under LWP,
+to allow an C<LWP::UserAgent> object to perform GET, HEAD and POST
+requests. Please see LWP for more information on POST requests.
+
+The C<Crypt::SSLeay> package provides C<Net::SSL>, which is loaded
+by C<LWP::Protocol::https> for https requests and provides the
+necessary SSL glue.
+
+This distribution also makes following deprecated modules available:
+
+  Crypt::SSLeay::CTX
+  Crypt::SSLeay::Conn
+  Crypt::SSLeay::X509
+
+Work on Crypt::SSLeay has been continued only to provide https
+support for the LWP (libwww-perl) libraries.
+
+=head1 ENVIRONMENT VARIABLES
+
+The following environment variables change the way
+C<Crypt::SSLeay> and C<Net::SSL> behave.
 
   # proxy support
   $ENV{HTTPS_PROXY} = 'http://proxy_hostname_or_ip:port';
@@ -76,64 +108,36 @@ Crypt::SSLeay - OpenSSL glue that provides LWP https support
   $ENV{HTTPS_PKCS12_FILE}     = 'certs/pkcs12.pkcs12';
   $ENV{HTTPS_PKCS12_PASSWORD} = 'PKCS12_PASSWORD';
 
-=head1 DESCRIPTION
-
-This document describes C<Crypt::SSLeay> version 0.53, released
-2006-12-26.
-
-This perl module provides support for the https protocol under LWP,
-to allow an C<LWP::UserAgent> object to perform GET, HEAD and POST
-requests. Please see LWP for more information on POST requests.
-
-The C<Crypt::SSLeay> package provides C<Net::SSL>, which is loaded
-by C<LWP::Protocol::https> for https requests and provides the
-necessary SSL glue.
-
-This distribution also makes following deprecated modules available:
-
-  Crypt::SSLeay::CTX
-  Crypt::SSLeay::Conn
-  Crypt::SSLeay::X509
-
-Work on Crypt::SSLeay has been continued only to
-provide https support for the LWP (libwww-perl)
-libraries. People wishing to access the OpenSSL API
-directly from Perl are advised to look at the
-C<Net::SSLeay> module.
-
-  http://search.cpan.org/dist/Net_SSLeay.pm/
-
 =head1 INSTALL
 
 =head2 OpenSSL
 
 You must have OpenSSL or SSLeay installed before compiling 
-this module.  You can get the latest OpenSSL package from:
+this module. You can get the latest OpenSSL package from:
 
   http://www.openssl.org/
 
 When installing openssl make sure your config looks like:
 
-  > ./config --openssldir=/usr/local/openssl
+  ./config --openssldir=/usr/local/openssl
  or
-  > ./config --openssldir=/usr/local/ssl
+  ./config --openssldir=/usr/local/ssl
 
-If you are planning on upgrading the default OpenSSL libraries on 
-a system like RedHat, not that I would recommend this, then 
-you might try something like:
+If you are planning on upgrading the default OpenSSL libraries on
+a system like RedHat, (not recommended), then try something like:
 
-  > ./config --openssldir=/usr --shared
+  ./config --openssldir=/usr --shared
 
 The --shared option to config will set up building the .so 
-shared libraries which is important for such systems.
+shared libraries which is important for such systems. This is
+followed by:
 
- then
-  > make
-  > make test
-  > make install
+  make
+  make test
+  make install
 
 This way Crypt::SSLeay will pick up the includes and 
-libraries automatically.  If your includes end up
+libraries automatically. If your includes end up
 going into a separate directory like /usr/local/include,
 then you may need to symlink /usr/local/openssl/include
 to /usr/local/include
@@ -141,35 +145,53 @@ to /usr/local/include
 =head2 Crypt::SSLeay
 
 The latest Crypt::SSLeay can be found at your nearest CPAN,
-and also:
+as well as:
 
   http://search.cpan.org/dist/Crypt-SSLeay/
 
 Once you have downloaded it, Crypt::SSLeay installs easily 
-using the make or nmake commands as shown below.  
+using the C<make> * commands as shown below.  
 
-  > perl Makefile.PL
-  > make
-  > make test
-  > make install
+  perl Makefile.PL
+  make
+  make test
+  make install
 
-  * use nmake for win32
+  * use nmake or dmake on Win32
 
-  !!! NOTE for Win32 users, few people seem to be able to build
-  W  Crypt::SSLeay successfully on that platform.  You don't need
-  I  to because ActiveState has already compiled it for you,
-  N  and is available for their perl builds 618 & 522 as a ppm
-  3  install.  It may also be available for their latest build.
-  2  For problems with this, please contact ActiveState.
-  !!!          Please see http://www.activestate.com/
+=head3 Windows
+
+C<Crypt::SSLeay> builds correctly with Strawberry Perl.
+
+For Activestate users, the ActiveState company does not have a
+permit from the Canadian Federal Government to distribute cryptographic
+software. This prevents C<Crypt::SSLeay> from being distributed as
+a PPM package from their repository. See
+L<http://aspn.activestate.com/ASPN/docs/ActivePerl/5.8/faq/ActivePerl-faq2.html#crypto_packages>
+for more information on this issue.
+
+You may download it from Randy Kobes's PPM repository by using
+the following command:
+
+  ppm install http://theoryx5.uwinnipeg.ca/ppms/Crypt-SSLeay.ppd
+
+An alternative is to add the uwinnipeg.ca PPM repository to your
+local installation. See L<http://cpan.uwinnipeg.ca/htdocs/faqs/ppm.html>
+for more details.
+
+=head3 VMS
+
+It is assumed that the OpenSSL installation is located at
+C</ssl$root>. Define this logical to point to the appropriate
+place in the filesystem.
 
 =head1 PROXY SUPPORT
 
 LWP::UserAgent and Crypt::SSLeay have their own versions of 
 proxy support. Please read these sections to see which one
-may be right for you.
+is appropriate.
 
-=head2 LWP::UserAgent Proxy Support
+=head2 LWP::UserAgent proxy support
 
 LWP::UserAgent has its own methods of proxying which may work for
 you and is likely to be incompatible with Crypt::SSLeay proxy support.
@@ -185,13 +207,13 @@ requests fine with an Apache mod_proxy server.  It sends a line like:
 
 to the proxy server, which is not the CONNECT request that
 some proxies would expect, so this may not work with other
-proxy servers than mod_proxy.  The CONNECT method is used
+proxy servers than mod_proxy. The CONNECT method is used
 by Crypt::SSLeay's internal proxy support.
 
-=head2 Crypt::SSLeay Proxy Support
+=head2 Crypt::SSLeay proxy support
 
 For native Crypt::SSLeay proxy support of https requests,
-you need to set an environment variable HTTPS_PROXY to your 
+you need to set the environment variable C<HTTPS_PROXY> to your 
 proxy server and port, as in:
 
   # proxy support
@@ -199,7 +221,7 @@ proxy server and port, as in:
   $ENV{HTTPS_PROXY} = '127.0.0.1:8080';
 
 Use of the C<HTTPS_PROXY> environment variable in this way 
-is similar to LWP::UserAgent->env_proxy() usage, but calling
+is similar to C<LWP::UserAgent->env_proxy()> usage, but calling
 that method will likely override or break the Crypt::SSLeay
 support, so do not mix the two.
 
@@ -210,9 +232,9 @@ this way:
   $ENV{HTTPS_PROXY_USERNAME} = 'username';
   $ENV{HTTPS_PROXY_PASSWORD} = 'password';  
 
-For an example of LWP scripting with Crypt::SSLeay native proxy
-support, please look at the F<lwp-ssl-test> script in the 
-Crypt::SSLeay distribution.
+For an example of LWP scripting with C<Crypt::SSLeay> native proxy
+support, please look at the F<eg/lwp-ssl-test> script in the 
+C<Crypt::SSLeay> distribution.
 
 =head1 CLIENT CERTIFICATE SUPPORT
 
@@ -222,11 +244,11 @@ private key files may be used like this:
   $ENV{HTTPS_CERT_FILE} = 'certs/notacacert.pem';
   $ENV{HTTPS_KEY_FILE}  = 'certs/notacakeynopass.pem';
 
-You may test your files with the F<net_ssl_test> program,
+You may test your files with the F<eg/net-ssl-test> program,
 bundled with the distribution, by issuing a command like:
 
-  ./net_ssl_test -cert=certs/notacacert.pem \
-	-key=certs/notacakeynopass.pem -d GET $HOST_NAME
+  perl eg/net-ssl-test -cert=certs/notacacert.pem \
+    -key=certs/notacakeynopass.pem -d GET $HOST_NAME
 
 Additionally, if you would like to tell the client where
 the CA file is, you may set these.
@@ -235,9 +257,9 @@ the CA file is, you may set these.
   $ENV{HTTPS_CA_DIR}  = "some_dir";
 
 There is no sample CA cert file at this time for testing,
-but you may configure ./net_ssl_test to use your CA cert
-with the -CAfile option. (TODO: then what is teh ./certs
-directory in the distribution??)
+but you may configure F<eg/net-ssl-test> to use your CA cert
+with the -CAfile option. (TODO: then what is the ./certs
+directory in the distribution?)
 
 =head2 Creating a test certificate
 
@@ -280,112 +302,70 @@ so you may set before using LWP or Net::SSL:
 
   $ENV{HTTPS_VERSION} = 3;
 
-so that a SSL v3 connection is tried first. At this time
-only a SSL v2 connection will be tried after this, as the 
-connection attempt order remains unchanged by this setting.
+to force a version 3 SSL connection first. At this time only a
+version 2 SSL connection will be tried after this, as the connection
+attempt order remains unchanged by this setting.
 
-=head1 BUILD NOTES
+=head1 ACKNOWLEDGEMENTS
 
-=head2 Win32, WinNT, Win2000, can't build
+Many thanks to Gisle Aas for writing this module and many others
+including libwww, for perl. The web will never be the same :)
 
-If you cannot get it to build on your windows box, try 
-ActiveState perl, at least their builds 522 & 618 are
-known to have a ppm install of Crypt::SSLeay available.
-Please see http://www.activestate.com for more info.
+Ben Laurie deserves kudos for his excellent patches for better error
+handling, SSL information inspection, and random seeding.
 
-=head2 AIX 4.3.2 - Symbol Error: __umoddi3 : referenced symbol not found
+Thanks to Dongqiang Bai for host name resolution fix when using a
+proxy.
 
-The __umoddi3 problem applies here as well when compiling with gcc.
+Thanks to Stuart Horner of Core Communications, Inc. who found the
+need for building --shared OpenSSL libraries.
 
-Alternative solution:
-In Makefile.PL, prepend C<-L>/usr/local/<path to your gcc lib>/<version>
-to the $LIBS value. Add after line 82:
+Thanks to Pavel Hlavnicka for a patch for freeing memory when using
+a pkcs12 file, and for inspiring more robust read() behavior.
 
- $LIBS = '-L' . dirname(`gcc -print-libgcc-file-name`) . ' ' . $LIBS;
-
-=head2 Solaris x86 - Symbol Error: __umoddi3 : referenced symbol not found
-
- Problem:
-
-On Solaris x86, the default PERL configuration, and preferred, is to use
-the ld linker that comes with the OS, not gcc.  Unfortunately during the 
-OpenSSL build process, gcc generates in libcrypto.a, from bn_word.c,
-the undefined symbol __umoddi3, which is supposed to be later resolved
-by gcc from libgcc.a
-
-The system ld linker does not know about libgcc.a by default, so 
-when building Crypt::SSLeay, there is a linker error for __umoddi3
-
- Solution:
-
-The fix for this symlink your libgcc.a to some standard directory
-like /usr/local/lib, so that the system linker, ld, can find
-it when building Crypt::SSLeay.  
-
-=head2 FreeBSD 2.x.x / Solaris - ... des.h:96 #error _ is defined ...
-
-If you encounter this error: "...des.h:96: #error _ is
-defined, but some strange definition the DES library cannot handle
-that...," then you need to edit the des.h file and comment out the 
-"#error" line.
-
-Its looks like this error might be common to other operating
-systems, and that occurs with OpenSSL 0.9.3.  Upgrades to
-0.9.4 seem to fix this problem.
-
-=head2 SunOS 4.1.4, Perl 5.004_04 - ld.so: Undefined symbol: _CRYPT_mem_ctrl
-
-Problems: (initial build was fine, but execution of Perl scripts had problems)
-
-Got a message "ld.so: Undefined symbol: _CRYPT_mem_ctrl"
-solution:  In the Makefile, comment out the line with
-"-fpic"  (also try changing to "-fPIC", and this works
-also, not sure if one is preferred).
-
-=head1 NOTES
-
-Many thanks to Gisle Aas for the original writing of 
-this module and many others including libwww for perl.  
-The web will never be the same :)
-
-Ben Laurie deserves kudos for his excellent patches
-for better error handling, SSL information inspection,
-and random seeding.
-
-Thanks to Dongqiang Bai for host name resolution fix when
-using a proxy.
-
-Thanks to Stuart Horner of Core Communications, Inc. who found
-the need for building --shared OpenSSL libraries.
-
-Thanks to Pavel Hlavnicka for a patch for freeing memory when
-using a pkcs12 file, and for inspiring more robust read() behavior.
-
-James Woodyatt is a champ for finding a ridiculous memory
-leak that has been the bane of many a Crypt::SSLeay user.
+James Woodyatt is a champ for finding a ridiculous memory leak that
+has been the bane of many a Crypt::SSLeay user.
 
 Thanks to Bryan Hart for his patch adding proxy support,
 and thanks to Tobias Manthey for submitting another approach.
 
 Thanks to Alex Rhomberg for Alpha linux ccc patch.
 
-Thanks to Tobias Manthey for his patches for client 
-certificate support.
-
-Thanks to Daisuke Kuroda for adding PKCS12 certificate
+Thanks to Tobias Manthey for his patches for client certificate
 support.
 
-Thanks to Gamid Isayev for CA cert support and 
-insights into error messaging.
+Thanks to Daisuke Kuroda for adding PKCS12 certificate support.
 
-Thanks to Jeff Long for working through a tricky CA
-cert SSLClientVerify issue.
+Thanks to Gamid Isayev for CA cert support and insights into error
+messaging.
+
+Thanks to Jeff Long for working through a tricky CA cert SSLClientVerify
+issue.
 
 Thanks to Chip Turner for patch to build under perl 5.8.0.
 
 Thanks to Joshua Chamas for the time he spent maintaining the
 module.
 
+Thanks to Jeff Lavallee for help with alarms on read failures (CPAN
+bug #12444).
+
+=head1 SEE ALSO
+
+=over 4
+
+=item Net::SSLeay
+
+A module that offers access to the OpenSSL API directly from Perl.
+
+  http://search.cpan.org/dist/Net_SSLeay.pm/
+
+=item http://www.openssl.org/related/binaries.html
+
+Pointers on where to find OpenSSL binary packages (Windows).
+
+=back
+
 =head1 SUPPORT
 
 For use of Crypt::SSLeay & Net::SSL with perl's LWP, please
@@ -406,7 +386,7 @@ This module is currently maintained by David Landgren.
 
 =head1 COPYRIGHT
 
- Copyright (c) 2006 David Landgren.
+ Copyright (c) 2006-2007 David Landgren.
  Copyright (c) 1999-2003 Joshua Chamas.
  Copyright (c) 1998 Gisle Aas.
 
